
#include <iostream>

using namespace std;

#include <boost/smart_ptr/intrusive_ptr.hpp>
#include "../src/nemo/IntrusiveCounted.h"
#include "../src/nemo/IntrusiveCowSupport.h"




template <class T>
#ifdef NO_GCC_ADL_WORKAROUND
class RefCounted : private nemo::IntrusiveCounted<RefCounted<T> > {
#else
class RefCounted : public nemo::IntrusiveCounted<RefCounted<T> > {
#endif
public:
	RefCounted(T val) : value(val) {}
	RefCounted(const RefCounted &r) : nemo::IntrusiveCounted<RefCounted<T> >(), value(r.value) {}

	// FIXME why does this have to be mutable???
	mutable T value;
};

template <class T>
class CopyOnWrite{
public:
	CopyOnWrite(T value) : member(new RefCounted<T>(value)) {}
	CopyOnWrite(CopyOnWrite<T> &r) : member(r.member) {}

	const T get() const {// read access
		return member->value;
	} 
	void set(T val){ // write access
		member.detach(); 
		member->value = val; 
	} 

private:
	nemo::IntrusiveCowSupport<RefCounted<T> > member;
};


int main(){

	typedef boost::intrusive_ptr<RefCounted<int> > IntPtr;

	IntPtr a(new RefCounted<int>(1));
	cout << a->value << endl;
	a->value = 3;
	cout << a->value << endl;

	CopyOnWrite<double> x(42.0);
	CopyOnWrite<double> y(x);
	cout << x.get() << endl;
	cout << y.get() << endl;
	y.set(21.0);
	cout << x.get() << endl;
	cout << y.get() << endl;
}

