
#include "../src/nemo/Mapping.h"

#include <iostream>

using namespace std;
using namespace nemo;


/*
 * A custom Mapping: boolean negation
 */
class Not : public MappingDefinition<bool,bool> {
public:
	static Mapping<bool,bool> create(){
		return Mapping<bool,bool>(new Not());
	}

	bool evaluate(bool value) const{
		return !value;
	}
};
/*
 * A custom Mapping: check whether an integer is even or not
 */
class IsEven : public MappingDefinition<int,bool> {
public:
	static Mapping<int,bool> create(){
		return Mapping<int,bool>(new IsEven());
	}

	bool evaluate(int value) const{
		return (value%2==0);
	}
};
/*
 * A custom Mapping: multiply double value by 2.0
 */
class TimesTwo : public MappingDefinition<double,double> {
public:
	static Mapping<double,double> create(){
		return Mapping<double,double>(new TimesTwo());
	}

	double evaluate(double value) const{
		return value*2.0;
	}
};
/*
 * A custom Mapping: multiply double value by 2.0
 */
class MinusOne : public MappingDefinition<double,double> {
public:
	static Mapping<double,double> create(){
		return Mapping<double,double>(new MinusOne());
	}

	double evaluate(double value) const{
		return value-1.0;
	}
};


int main(){
	typedef Mapping<bool,bool> BooleanMapping;

	bool trueValue = true;

	// use a built-in mapping
	BooleanMapping id = Identity<bool>::create();
	bool alsoTrue = id(trueValue);

	// use a custom mapping
	BooleanMapping negate = Not::create();
	bool falseValue = negate(trueValue);

	// chain different functions. negId(foo) evaluates to id(negate(foo))
	BooleanMapping negId = id(negate);
	bool alsoFalse = negId(trueValue);

	cout << trueValue << "\t" << alsoTrue << "\t" << falseValue << "\t" << alsoFalse << endl;

	int i=2, j=3, k=4;

	// use custom mixed-type mappings
	Mapping<int,bool> isEven = IsEven::create();
	cout << isEven(i) << "\t" << isEven(j) << "\t" << isEven(k) << endl;

	// ... and chain them!
	Mapping<int,bool> isOdd = negate(isEven);
	cout << isOdd(i) << "\t" << isOdd(j) << "\t" << isOdd(k) << endl;


	// chain numeric functions
	typedef Mapping<double,double> RealMapping;
	RealMapping timesTwo = TimesTwo::create();
	RealMapping minusOne = MinusOne::create();

	RealMapping someMapping = timesTwo(minusOne); // y = 2*(x-1) 

	cout << someMapping(1.0) << endl; // =0.0
	cout << someMapping(2.0) << endl; // =2.0


}


