#include "../src/nemo/Mapping.h"
#include "../src/nemo/MatrixMath.h" // for inversion of Matrix*arg<Vector> 

#include <iostream>

using namespace std;
using namespace nemo;


int main(){

	typedef Mapping<double,double> RealMapping;

	// redefine custom mappings from ExampleMapping.cpp with literals and numeric operators
	RealMapping timesTwo = 2.0 * arg<double>();
	RealMapping minusOne = arg<double>() - 1.0;

	RealMapping someMapping = timesTwo(minusOne); // y = 2*(x-1)

	cout << someMapping(1.0) << endl; // =0.0
	cout << someMapping(2.0) << endl; // =2.0
	
	// use operators on mappings
	RealMapping someOtherMapping = timesTwo + minusOne; // y = (2*x) + (x-1) 
	
	cout << someOtherMapping(1.0) << endl; // =2.0
	cout << someOtherMapping(2.0) << endl; // =5.0

	// add mappings and constants
	RealMapping yetAnotherMapping = 1.0 / someOtherMapping; // y = 1.0 / [(2*x) + (x-1)]
	
	cout << yetAnotherMapping(1.0) << endl; // =0.5
	cout << yetAnotherMapping(2.0) << endl; // =0.2


	// another custom mapping from ExampleMapping.cpp, expressed with operators
	Mapping<int,bool> isEven = (arg<int>() % 2)==0;

	cout << isEven(0) << endl; // true
	cout << isEven(1) << endl; // false
	cout << isEven(2) << endl; // true

	// cast functions
	Mapping<double,int> round = reinterpret<int>(arg<double>());
	cout << "(int)2.5 = " << round(2.5) << endl;

	
	// Invert mappings easily:
	RealMapping divByTwo = timesTwo.inverse();
	cout << divByTwo(0) << endl; // =0.0
	cout << divByTwo(1) << endl; // =0.5
	cout << divByTwo(2) << endl; // =1.0

	RealMapping foo = 2.0 * (arg<double>() / 4.0 - 13.0);
	RealMapping fooInv = foo.inverse();
	cout << "0 \t" << foo(0) << "\t" << fooInv(foo(0)) << endl;
	cout << "1 \t" << foo(1) << "\t" << fooInv(foo(1)) << endl;
	cout << "2 \t" << foo(2) << "\t" << fooInv(foo(2)) << endl;

	// ... also when vectors and matrices are involved:
	Mapping<RealVector,RealVector> linear = 
		RealMatrix::diagonal(RealVector(2.0, 3.0)) * arg<RealVector>() + RealVector(1.0, 0.5);
	Mapping<RealVector,RealVector> linearInv = linear.inverse();
	cout << RealVector(1.0, 2.0) << endl;
	cout << linear(RealVector(1.0, 2.0)) << endl;
	cout << linearInv(linear(RealVector(1.0, 2.0))) << endl; // [1.0, 2.0]
	
	
}


