
#include "TimeSeries.h"
#include "Vector.h"
#include "Matrix.h"
#include "Mapping.h"

#include <iostream>

using namespace nemo;
using namespace std;


int main(){
	TimeSeries<int,RealVector> series;
	// fill subsequent data
	series.append(2, RealVector(1.0, 2.0));
	series.append(4, RealVector(3.0, 2.0));
	cout << series << endl;

	// access and modify data by timestamp->value association
	series.setValue(2, RealVector(1.0, 5.0));
	cout << "2" << "\t" << series[2] << endl;

	// ... or by index-based access
	cout << series.timestamps[0] << "\t" << series.values[0] << endl << endl;

	
	// use different timestamp types
	TimeSeries<double,RealVector> realSeries;
	realSeries.append(1.5, RealVector(1.0, 1.0));
	realSeries.append(2.5, RealVector(3.0, 1.0));
	cout << realSeries << endl;

	// insert new values
	realSeries.insert(2.1, RealVector(2.0, 2.0));
	cout << realSeries << endl;

	
	// transform values (e.g. exchange vector components and scale them)
#ifdef _NEMO_HAS_INIT_LISTS_
	TimeSeries<double,RealVector> transformed = realSeries.values.map(
		RealMatrix({{0,2},{1,0}}) * arg<RealVector>()
	);
#else
	RealMatrix mat(dim(2,2));
	mat.setRow(0, RealVector(0.0, 2.0).transpose());
	mat.setRow(1, RealVector(1.0, 0.0).transpose());
	TimeSeries<double,RealVector> transformed = realSeries.values.map(
		mat * arg<RealVector>()
	);
#endif
	cout << transformed << endl;

	// transform timestamps (e.g. reverse the series)
	transformed.timestamps.transform( 2.5 - arg<double>() );
	cout << transformed << endl;

}

