
#include <iostream>
#include <algorithm>

#include "Vector.h"

using namespace std;
using namespace nemo;

int main(){
	/// construction
	// by dimension and initial value per component
	MathVector<int> i(dim(2), 42);
	RealVector x(dim(3), 21.0); // RealVector is a typedef on MathVector<double>
#ifdef _NEMO_HAS_INIT_LISTS_
	// ... or with initializer lists (C++11 syntax)
	RealVector s = {1.0,2.0,3.0,4.0,5.0}; // five-dimensional vector
#endif
	// ... or directly with up to four values
	RealVector t(1.0,2.0,3.0); // three-dimensional vector
	
	/// Access and manipulation
	cout << x[0] << endl; // output: 21
	x[1] = 3.0; 
	cout << x[1] << endl; // output: 3

	/// Memory and runtime efficient "copy-on-write":
	RealVector y = x; // no deep-copy here
	cout << y[0] << endl; // output: 21
	// this is where the copy is internally made:
	y[0] = 2.0; 
	cout << y[0] << endl; // output: 2.0
	
	/// Standard arithmetic operators
	RealVector z = 2 * (x - y);
	cout << x << endl; // output: [21, 3, 21]
	cout << y << endl; // output: [2, 3, 21]
	cout << z << endl; // output: [38, 0, 0]

	/// Type-safe scalar-product:
	double l = t.transpose() * t;
	cout << l << endl; // output: 14  (which is 1*1+2*2+3*3)

	/// Component-wise vector-vector operations
	z = x.cwise() / y;
	cout << z << endl; // output: [10.5, 1, 1]
	z *= t.cwise();
	cout << z << endl; // output: [10.5, 2, 3]

	/// Flexible component-wise operations with nemo::Mappings
	RealVector a(1.0, 0.0, 2.0);
	a.transform( 2.0 - arg<double>() ); // each component a[i] is set to 2.0-a[i]
	cout << a << endl; // output: [1, 2, 0]

	/// Full STL-compliance:
	std::sort(a.begin(), a.end()); // let STL use MathVector's iterators 
	cout << a << endl; // output: [0, 1, 2]
}

