#ifndef _NEMO_NORMS_H_
#define _NEMO_NORMS_H_

#include "Mapping.h"
#include "detail/nemo-cpp11.h"

#include <cmath>


namespace nemo {

#ifndef NEMO_PARSED_BY_DOXYGEN
template <class T> 
class AbsNormDefinition : public MappingDefinition<T,T> {
public:
	T evaluate(T value) const {
		return (value<0)?(-value):value;
	}
};
#endif

template <typename T> 
static inline Mapping<T,T> absNorm(){
	return Mapping<T,T>(new AbsNormDefinition<T>());
}

#ifndef NEMO_PARSED_BY_DOXYGEN
template <class T, class NormType> 
class EuclideanNormDefinition : public MappingDefinition<T,NormType> {
public:
	NormType evaluate(T value) const {
		NormType squareSum = (NormType)0;
		for(unsigned int i=0; i<value.numElements(); i++){
			squareSum += value.getElement(i)*value.getElement(i);
		}
		return sqrt(squareSum);
	}
};
#endif

#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
template <typename T, typename NormType=double> 
#else
template <typename T, typename NormType> 
#endif
static inline Mapping<T,NormType> euclideanNorm(){
	return Mapping<T,NormType>(new EuclideanNormDefinition<T,NormType>());
}


#ifndef NEMO_PARSED_BY_DOXYGEN
template <class T, class NormType=double> 
class EuclideanSquareDefinition : public MappingDefinition<T,NormType> {
public:
	NormType evaluate(T value) const {
		NormType squareSum = (NormType)0;
		for(unsigned int i=0; i<value.numElements(); i++){
			squareSum += value.getElement(i)*value.getElement(i);
		}
		return squareSum;
	}
};
#endif

#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
template <typename T, typename NormType=double> 
#else
template <typename T, typename NormType> 
#endif
static inline Mapping<T,NormType> euclideanSquare(){
	return Mapping<T,NormType>(new EuclideanSquareDefinition<T,NormType>());
}


#ifndef NEMO_PARSED_BY_DOXYGEN
template <class T, class NormType=double> 
class MaxNormDefinition : public MappingDefinition<T,NormType> {
public:
	NormType evaluate(T value) const {
		NormType max = (NormType)0;
		for(unsigned int i=0; i<value.numElements(); i++){
			if(value.getElement(i) < -max){
				max = -value.getElement(i);
			}
			else if(max < value.getElement(i)){
				max = value.getElement(i);
			}
		}
		return max;
	}
};
#endif

#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
template <typename T, typename NormType=double> 
#else
template <typename T, typename NormType> 
#endif
static inline Mapping<T,NormType> maxNorm(){
	return Mapping<T,NormType>(new MaxNormDefinition<T,NormType>());
}


} // end namespace

#endif

