


/*! \mainpage NemoMath Overview
 *
 * \section Introduction
 *
 * A header-only template library for mathematical computations in the context of robotics and learning.
 *  - Matrix and Vector data types, optimized for shared-ownership in object-oriented applications by means of "Copy on write" and intrusive-pointering
 *  - Neat functional programming API to generate and use Functor objects
 * 
 *
 * \section la Linear Algebra data types
 * 
 * The MathVector and Matrix classes provide standard linear algebra functionality.
 * They are optimized for shared ownership in complex object-oriented application.
 * Therefore they comprise a combination of efficient intrusive-pointer mechanisms and copy-on write.
 * @see Vector.h Matrix.h
 * @see MatrixMath.h
 * 
 * \section func Functor library
 * 
 * NemoMath provides a very convenient way to express and utilize mathematical function objects (functors).
 * You can define your own function objects or just use built in operators to express complex mathematical
 * relations. The Collection API allows these functors to smoothly interoperate with the Matrix and Vector data types.
 * @see Mapping.h Collection.h
 * @see ScalarMappings.h VectorMappings.h Norms.h Filters.h
 * 
 * \section cont Other mathematical data-structures
 * 
 * NemoMath provides container data types for more complex structures. Currently it provides an 
 * abstraction for time series.
 * @see TimeSeries.h
 * 
 * \section cont Utilities
 * 
 * Convenient and generic random number generators:
 * @see Random.h
 * 
 * Tools for efficient memory management:
 * @see IntrusivePtr.h IntrusiveCounted.h IntrusiveCowSupport.h
 *  
 */
