
#define BOOST_TEST_MODULE MathVectorCollectionAPI
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Vector.h"

#include <cmath>

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE(MathVectorCollectionAPI)


BOOST_AUTO_TEST_CASE(mathVectorAccess) {
	RealVector v1;
	BOOST_CHECK_EQUAL(v1.numElements(), 0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(v1.getElement(0), std::invalid_argument);
	BOOST_CHECK_THROW(v1.setElement(0, 1.0), std::invalid_argument);
#endif

	RealVector v2(1.0, 2.0);
	BOOST_CHECK_EQUAL(v2.numElements(), 2);
	BOOST_CHECK_EQUAL(v2.getElement(0), 1.0);
	BOOST_CHECK_EQUAL(v2.getElement(1), 2.0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(v2.getElement(2), std::invalid_argument);
#endif

	BOOST_CHECK_NO_THROW(v2.setElement(0, 3.0));
	BOOST_CHECK_NO_THROW(v2.setElement(1, 4.0));
	BOOST_CHECK_EQUAL(v2.numElements(), 2);
	BOOST_CHECK_EQUAL(v2.getElement(0), 3.0);
	BOOST_CHECK_EQUAL(v2.getElement(1), 4.0);
}

BOOST_AUTO_TEST_CASE(mathVectorTransformMap) {
	RealVector v1;
	v1.transform( arg<double>() + 1.0 );
	BOOST_CHECK_EQUAL(v1.numElements(), 0);

	RealVector v2 = v1.map(arg<double>() + 1.0);
	BOOST_CHECK_EQUAL(v2.numElements(), 0);

	RealVector v3(1.0, 2.0);
	RealVector v3B(1.0, 2.0);
	v3.transform(arg<double>() + 2.0);
	BOOST_CHECK_EQUAL(v3.numElements(), 2);
	BOOST_CHECK_EQUAL(v3.getElement(0), 3.0);
	BOOST_CHECK_EQUAL(v3.getElement(1), 4.0);
	BOOST_CHECK_EQUAL(v3B.getElement(0), 1.0); // unchanged?
	BOOST_CHECK_EQUAL(v3B.getElement(1), 2.0); // unchanged?

	const RealVector v3c =v3;
	RealVector v4 = v3.map(arg<double>() + 2.0);
	BOOST_CHECK_EQUAL(v4.numElements(), 2);
	BOOST_CHECK_EQUAL(v4.getElement(0), 5.0);
	BOOST_CHECK_EQUAL(v4.getElement(1), 6.0);
	BOOST_CHECK_EQUAL(v3c.getElement(0), 3.0); // unchanged?
	BOOST_CHECK_EQUAL(v3c.getElement(1), 4.0); // unchanged?
}

BOOST_AUTO_TEST_SUITE_END()


