
#define BOOST_TEST_MODULE MathVectorDimChange
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Vector.h"

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE(MathVectorDimChange)


BOOST_AUTO_TEST_CASE(mathVectorAttach) {
	RealVector vDim0;
	RealVector vDim0B(vDim0);
	BOOST_CHECK_EQUAL( (vDim0.attach(vDim0)).dimension(), 0 );

	RealVector v1(1.0);
	RealVector v1B(v1);
	RealVector v2(2.0, 2.0);
	RealVector v2B(v2);
	RealVector v3;

	v3 = vDim0.attach(v2);
	BOOST_CHECK_EQUAL( v3, v2 );
	BOOST_CHECK_EQUAL( vDim0, vDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?

	v3 = v2.attach(vDim0);
	BOOST_CHECK_EQUAL( v3, v2 );
	BOOST_CHECK_EQUAL( vDim0, vDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?

	v3 = v1.attach(v2);
	BOOST_CHECK_EQUAL( v3, RealVector(1.0, 2.0, 2.0) );
	BOOST_CHECK_EQUAL( v1, v1B ); // unchanged?
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?

	v3 = v2.attach(v1);
	BOOST_CHECK_EQUAL( v3, RealVector(2.0, 2.0, 1.0) );
	BOOST_CHECK_EQUAL( v1, v1B ); // unchanged?
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?
}

BOOST_AUTO_TEST_CASE(mathVectorSubVector) {
	RealVector vDim0;
	RealVector vDim0B(vDim0);
	
	RealVector vDim0x = vDim0.subVector(0, dim(0));
	BOOST_CHECK_EQUAL( vDim0x, vDim0B );
	BOOST_CHECK_EQUAL( vDim0, vDim0B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( vDim0.subVector(1, dim(0)), std::invalid_argument );
	BOOST_CHECK_THROW( vDim0.subVector(0, dim(1)), std::invalid_argument );
	BOOST_CHECK_EQUAL( vDim0, vDim0B ); // unchanged?
#endif

	RealVector v1(1.0);
	RealVector v1B(v1);
	RealVector v2 = RealVector(2.0, 3.0);
	RealVector v2B(v2);
	RealVector v3;

	v3 = v2.subVector(0, dim(0));
	BOOST_CHECK_EQUAL( v3, vDim0 );
	v3 = v2.subVector(1, dim(0));
	BOOST_CHECK_EQUAL( v3, vDim0 );
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( v2.subVector(2, dim(0)), std::invalid_argument );
	BOOST_CHECK_THROW( v2.subVector(0, dim(3)), std::invalid_argument );
#endif

	v3 = v2.subVector(0, dim(1));
	BOOST_CHECK_EQUAL( v3, RealVector(2.0) );
	v3 = v2.subVector(1, dim(1));
	BOOST_CHECK_EQUAL( v3, RealVector(3.0) );
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?

	v3 = v2.subVector(0, dim(2));
	BOOST_CHECK_EQUAL( v3, v2 );
	BOOST_CHECK_EQUAL( v2, v2B ); // unchanged?
}

BOOST_AUTO_TEST_SUITE_END()

