
#define BOOST_TEST_MODULE MatrixCollectionAPI
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Matrix.h"

#include <cmath>

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE(MatrixCollectionAPI)


BOOST_AUTO_TEST_CASE(matrixAccess) {
	RealMatrix m1;
	BOOST_CHECK_EQUAL(m1.numElements(), 0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m1.getElement(0), std::invalid_argument);
	BOOST_CHECK_THROW(m1.setElement(0, 1.0), std::invalid_argument);
#endif

	RealMatrix m2(dim(2,3));
	m2.setRow(0, RealVector(1.0, 2.0, 3.0).transpose());
	m2.setRow(1, RealVector(4.0, 5.0, 6.0).transpose());
	BOOST_CHECK_EQUAL(m2.numElements(), 6);
	BOOST_CHECK_EQUAL(m2.getElement(0), 1.0);
	BOOST_CHECK_EQUAL(m2.getElement(1), 2.0);
	BOOST_CHECK_EQUAL(m2.getElement(2), 3.0);
	BOOST_CHECK_EQUAL(m2.getElement(3), 4.0);
	BOOST_CHECK_EQUAL(m2.getElement(4), 5.0);
	BOOST_CHECK_EQUAL(m2.getElement(5), 6.0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m2.getElement(6), std::invalid_argument);
#endif

	BOOST_CHECK_NO_THROW(m2.setElement(0, 7.0));
	BOOST_CHECK_NO_THROW(m2.setElement(5, 8.0));
	BOOST_CHECK_EQUAL(m2.numElements(), 6);
	BOOST_CHECK_EQUAL(m2.getElement(0), 7.0);
	BOOST_CHECK_EQUAL(m2.getElement(5), 8.0);
}

BOOST_AUTO_TEST_CASE(mathVectorTransformMap) {
	RealMatrix m1;
	m1.transform( arg<double>() + 1.0 );
	BOOST_CHECK_EQUAL(m1.numElements(), 0);

	RealMatrix m11 = m1.map(arg<double>() + 1.0);
	BOOST_CHECK_EQUAL(m11.numElements(), 0);

	RealMatrix m2(dim(2,3));
	m2.setRow(0, RealVector(1.0, 2.0, 3.0).transpose());
	m2.setRow(1, RealVector(4.0, 5.0, 6.0).transpose());
	RealMatrix m2B(dim(2,3));
	m2B.setRow(0, RealVector(1.0, 2.0, 3.0).transpose());
	m2B.setRow(1, RealVector(4.0, 5.0, 6.0).transpose());

	m2.transform(arg<double>() + 6.0);
	BOOST_CHECK_EQUAL(m2.numElements(), 6);
	BOOST_CHECK_EQUAL(m2.getElement(0), 7.0);
	BOOST_CHECK_EQUAL(m2.getElement(1), 8.0);
	BOOST_CHECK_EQUAL(m2.getElement(2), 9.0);
	BOOST_CHECK_EQUAL(m2.getElement(3), 10.0);
	BOOST_CHECK_EQUAL(m2.getElement(4), 11.0);
	BOOST_CHECK_EQUAL(m2.getElement(5), 12.0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m2.getElement(6), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL(m2B.getElement(0), 1.0); // unchanged?
	BOOST_CHECK_EQUAL(m2B.getElement(1), 2.0); // unchanged?

	RealMatrix m4 = m2.map(arg<double>() + 6.0);
	BOOST_CHECK_EQUAL(m4.numElements(), 6);
	BOOST_CHECK_EQUAL(m4.getElement(0), 13.0);
	BOOST_CHECK_EQUAL(m4.getElement(1), 14.0);
	BOOST_CHECK_EQUAL(m4.getElement(2), 15.0);
	BOOST_CHECK_EQUAL(m4.getElement(3), 16.0);
	BOOST_CHECK_EQUAL(m4.getElement(4), 17.0);
	BOOST_CHECK_EQUAL(m4.getElement(5), 18.0);
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m4.getElement(6), std::invalid_argument);
#endif
}

BOOST_AUTO_TEST_SUITE_END()


