
#define BOOST_TEST_MODULE MatrixDimChance
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Matrix.h"

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;

BOOST_AUTO_TEST_SUITE(MatrixDimChance)


BOOST_AUTO_TEST_CASE(matrixAttachRows) {
	RealMatrix mDim0;
	RealMatrix mDim0B(mDim0);
	BOOST_CHECK_EQUAL( (mDim0.attachRows(mDim0)).dimension(), dim(0,0) );

	RealMatrix m0(dim(0,1));
	RealMatrix m0B(m0);
	RealMatrix m1(dim(1,1), 1.0);
	RealMatrix m1B(m1);
	RealMatrix m2(dim(2,1), 2.0);
	RealMatrix m2B(m2);
	RealMatrix m3;

	m3 = m0.attachRows(m2);
	BOOST_CHECK_EQUAL( m3, m2 );
	BOOST_CHECK_EQUAL( m0, m0B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m2.attachRows(m0);
	BOOST_CHECK_EQUAL( m3, m2 );
	BOOST_CHECK_EQUAL( m0, m0B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m1.attachRows(m2);
	RealMatrix m1m2(dim(3,1));
	m1m2(0,0) = 1.0; m1m2(1,0) = 2.0;  m1m2(2,0) = 2.0; 
	BOOST_CHECK_EQUAL( m3, m1m2 );
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m2.attachRows(m1);
	RealMatrix m2m1(dim(3,1));
	m2m1(0,0) = 2.0; m2m1(1,0) = 2.0;  m2m1(2,0) = 1.0; 
	BOOST_CHECK_EQUAL( m3, m2m1 );
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	// dimension mismatch
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(mDim0.attachRows(m1), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m1.attachRows(mDim0), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m1.attachRows(RealMatrix(dim(2,2))), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
}

BOOST_AUTO_TEST_CASE(matrixAttachCols){
	RealMatrix mDim0;
	RealMatrix mDim0B(mDim0);
	BOOST_CHECK_EQUAL( (mDim0.attachCols(mDim0)).dimension(), dim(0,0) );

	RealMatrix m0(dim(1,0));
	RealMatrix m0B(m0);
	RealMatrix m1(dim(1,1), 1.0);
	RealMatrix m1B(m1);
	RealMatrix m2(dim(1,2), 2.0);
	RealMatrix m2B(m2);
	RealMatrix m3;

	m3 = m0.attachCols(m2);
	BOOST_CHECK_EQUAL( m3, m2 );
	BOOST_CHECK_EQUAL( m0, m0B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m2.attachCols(m0);
	BOOST_CHECK_EQUAL( m3, m2 );
	BOOST_CHECK_EQUAL( m0, m0B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m1.attachCols(m2);
	RealMatrix m1m2(dim(1,3));
	m1m2(0,0) = 1.0; m1m2(0,1) = 2.0;  m1m2(0,2) = 2.0; 
	BOOST_CHECK_EQUAL( m3, m1m2 );
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m2.attachCols(m1);
	RealMatrix m2m1(dim(1,3));
	m2m1(0,0) = 2.0; m2m1(0,1) = 2.0;  m2m1(0,2) = 1.0; 
	BOOST_CHECK_EQUAL( m3, m2m1 );
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	// dimension mismatch
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(mDim0.attachCols(m1), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m1.attachCols(mDim0), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW(m1.attachCols(RealMatrix(dim(2,2))), std::invalid_argument);
#endif
	BOOST_CHECK_EQUAL( m1, m1B ); // unchanged?
}

BOOST_AUTO_TEST_CASE(matrixSubMatrix) {
	RealMatrix mDim0;
	RealMatrix mDim0B(mDim0);
	
	RealMatrix mDim0x = mDim0.subMatrix(0, 0, dim(0,0));
	BOOST_CHECK_EQUAL( mDim0x, mDim0B );
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( mDim0.subMatrix(0, 0, dim(0, 1)), std::invalid_argument );
	BOOST_CHECK_THROW( mDim0.subMatrix(0, 0, dim(1, 0)), std::invalid_argument );
	BOOST_CHECK_THROW( mDim0.subMatrix(0, 1, dim(0, 0)), std::invalid_argument );
	BOOST_CHECK_THROW( mDim0.subMatrix(1, 0, dim(0, 0)), std::invalid_argument );
#endif
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?

	RealMatrix m1(dim(1,1), 1.0);
	RealMatrix m1B(m1);
	RealMatrix m2(dim(2,1));
	m2(0,0) = 2.0;
	m2(1,0) = 3.0;
	RealMatrix m2B(m2);
	RealMatrix m3;

	m3 = m2.subMatrix(0, 0, dim(0,0));
	BOOST_CHECK_EQUAL( m3, mDim0 );
	m3 = m2.subMatrix(1, 0, dim(0,0));
	BOOST_CHECK_EQUAL( m3, mDim0 );
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	BOOST_CHECK_EQUAL( m2.subMatrix(2, 0, dim(0, 1)).dimension(), dim(0, 1) );

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( m2.subMatrix(0, 0, dim(3, 1)), std::invalid_argument );
#endif

	m3 = m2.subMatrix(0, 0, dim(1,1));
	BOOST_CHECK_EQUAL( m3, RealMatrix(dim(1,1), 2.0) );
	m3 = m2.subMatrix(1, 0, dim(1,1));
	BOOST_CHECK_EQUAL( m3, RealMatrix(dim(1,1), 3.0) );
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?

	m3 = m2.subMatrix(0, 0, dim(2,1));
	BOOST_CHECK_EQUAL( m3, m2 );
	BOOST_CHECK_EQUAL( m2, m2B ); // unchanged?
}

BOOST_AUTO_TEST_CASE(matrixTranspose) {
	RealMatrix mDim0;
	RealMatrix mDim0B(mDim0);
	
	RealMatrix mDim0x = mDim0.transpose();
	BOOST_CHECK_EQUAL( mDim0x, mDim0B );
	BOOST_CHECK_EQUAL( mDim0, mDim0B ); // unchanged?

	RealMatrix mDim01(dim(0,1));
	RealMatrix mDim10(dim(1,0));
	BOOST_CHECK_EQUAL( mDim01.transpose().dimension(), dim(1,0) );
	BOOST_CHECK_EQUAL( mDim01.transpose(), mDim10 );
	BOOST_CHECK_EQUAL( mDim01.dimension(), dim(0,1) ); // unchanged?
	BOOST_CHECK_EQUAL( mDim10.transpose().dimension(), dim(0,1) );
	BOOST_CHECK_EQUAL( mDim10.transpose(), mDim01 );
	BOOST_CHECK_EQUAL( mDim10.dimension(), dim(1,0) ); // unchanged?


	RealMatrix m1(dim(1,1), 1.0);
	RealMatrix m1B(m1);
	RealMatrix m2(dim(2,1));
	m2(0,0) = 2.0;
	m2(1,0) = 3.0;
	RealMatrix m2B(m2);
	RealMatrix m2T(dim(1,2));
	m2T(0,0) = 2.0;
	m2T(0,1) = 3.0;

	BOOST_CHECK_EQUAL( m1.transpose(), m1 );
	BOOST_CHECK_EQUAL( m2.transpose(), m2T );
	BOOST_CHECK_EQUAL( m2T.transpose(), m2 );

	const RealMatrix m2c = m2;
	BOOST_CHECK_EQUAL( m2c.transpose(), m2T );
	
}

BOOST_AUTO_TEST_SUITE_END()

