
#define BOOST_TEST_MODULE MathVectorIO
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Matrix.h"

#include <string>
#include <sstream>

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;
using namespace std;


BOOST_AUTO_TEST_SUITE(MathVectorIO)


BOOST_AUTO_TEST_CASE(mathVectorOutStreamZeroDim){
	RealMatrix m00;
	stringstream s00;
	s00 << m00;
	BOOST_CHECK_EQUAL( s00.str(), string("[]") );

	RealMatrix m01(dim(0,1));
	stringstream s01expect;
	s01expect << "[]";
	stringstream s01;
	s01 << m01;
	BOOST_CHECK_EQUAL( s01.str(), s01expect.str() );
	
	RealMatrix m10(dim(1,0));
	stringstream s10expect;
	s10expect << "[]";
	stringstream s10;
	s10 << m10;
	BOOST_CHECK_EQUAL( s10.str(), s10expect.str() );
	
	RealMatrix m20(dim(2,0));
	stringstream s20expect;
	s20expect << "[]";
	stringstream s20;
	s20 << m20;
	BOOST_CHECK_EQUAL( s20.str(), s20expect.str() );
}

BOOST_AUTO_TEST_CASE(mathVectorOutStream){
	
	RealMatrix m21(dim(2,1));
	m21.setCol(0, RealVector(1.0, 2.0));
	stringstream s21expect;
	s21expect << "[1;" << endl << " 2]";
	stringstream s21;
	s21 << m21;
	BOOST_CHECK_EQUAL( s21.str(), s21expect.str() );
	
	RealMatrix m12(dim(1,2));
	m12.setRow(0, RealVector(1.0, 2.0).transpose());
	stringstream s12expect;
	s12expect << "[1,\t2]";
	stringstream s12;
	s12 << m12;
	BOOST_CHECK_EQUAL( s12.str(), s12expect.str() );
	
	RealMatrix m22(dim(2,2));
	m22.setRow(0, RealVector(1.0, 2.0).transpose());
	m22.setRow(1, RealVector(3.0, 4.0).transpose());
	stringstream s22expect;
	s22expect << "[1,\t2;" << endl << " 3,\t4]";
	stringstream s22;
	s22 << m22;
	BOOST_CHECK_EQUAL( s22.str(), s22expect.str() );
}


BOOST_AUTO_TEST_SUITE_END()

