
#define BOOST_TEST_MODULE MatrixScalarArithmetics
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Matrix.h"

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE(MatrixScalarArithmetics)


BOOST_AUTO_TEST_CASE(matrixScalarMultRight){
	// zero dim matrices
	RealMatrix mDim0;
	BOOST_CHECK_EQUAL( (mDim0 * 2.0).dimension(), dim(0,0) );
	RealMatrix mDim01(dim(0,1));
	BOOST_CHECK_EQUAL( (mDim01 * 2.0).dimension(), dim(0,1) );
	RealMatrix mDim10(dim(1,0));
	BOOST_CHECK_EQUAL( (mDim10 * 2).dimension(), dim(1,0) );

	// square matrices
	RealMatrix m1(dim(2,2), 1.0);
	RealMatrix m1B = m1;
	BOOST_CHECK_EQUAL( m1 * 2.0, RealMatrix(dim(2,2), 2.0) );
	BOOST_CHECK_EQUAL( m1B, RealMatrix(dim(2,2), 1.0) );

	// rect matrices
	RealMatrix m3(dim(1,2), 1.0);
	RealMatrix m3B = m3;
	BOOST_CHECK_EQUAL( (m3 * 2.0), RealMatrix(dim(1,2), 2.0) );
	BOOST_CHECK_EQUAL( m3B, RealMatrix(dim(1,2), 1.0) );
}

BOOST_AUTO_TEST_CASE(matrixScalarMultLeft){
	// zero dim matrices
	RealMatrix mDim0;
	BOOST_CHECK_EQUAL( (2.0 * mDim0).dimension(), dim(0,0) );
	RealMatrix mDim01(dim(0,1));
	BOOST_CHECK_EQUAL( (2.0 * mDim01).dimension(), dim(0,1) );
	RealMatrix mDim10(dim(1,0));
	BOOST_CHECK_EQUAL( (2.0 * mDim10).dimension(), dim(1,0) );

	// square matrices
	RealMatrix m1(dim(2,2), 1.0);
	RealMatrix m1B = m1;
	BOOST_CHECK_EQUAL( 2.0 * m1, RealMatrix(dim(2,2), 2.0) );
	BOOST_CHECK_EQUAL( m1B, RealMatrix(dim(2,2), 1.0) );

	// rect matrices
	RealMatrix m3(dim(1,2), 1.0);
	RealMatrix m3B = m3;
	BOOST_CHECK_EQUAL( (2.0 * m3), RealMatrix(dim(1,2), 2.0) );
	BOOST_CHECK_EQUAL( m3B, RealMatrix(dim(1,2), 1.0) );
}

BOOST_AUTO_TEST_CASE(matrixScalarMultInplace){
	// zero dim matrices
	RealMatrix mDim0;
	mDim0 *= 2.0;
	BOOST_CHECK_EQUAL( mDim0.dimension(), dim(0,0) );
	RealMatrix mDim01(dim(0,1));
	mDim01 *= 2.0;
	BOOST_CHECK_EQUAL( mDim01.dimension(), dim(0,1) );
	RealMatrix mDim10(dim(1,0));
	mDim10 *= 2.0;
	BOOST_CHECK_EQUAL( mDim10.dimension(), dim(1,0) );

	// square matrices
	RealMatrix m1(dim(2,2), 1.0);
	RealMatrix m1B = m1;
	m1 *= 2.0;
	BOOST_CHECK_EQUAL( m1, RealMatrix(dim(2,2), 2.0) );
	BOOST_CHECK_EQUAL( m1B, RealMatrix(dim(2,2), 1.0) );

	// rect matrices
	RealMatrix m3(dim(1,2), 1.0);
	RealMatrix m3B = m3;
	m3 *= 2.0;
	BOOST_CHECK_EQUAL( m3, RealMatrix(dim(1,2), 2.0) );
	BOOST_CHECK_EQUAL( m3B, RealMatrix(dim(1,2), 1.0) );
}

BOOST_AUTO_TEST_SUITE_END()

