
#define BOOST_TEST_MODULE MatrixVectorArithmetics
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Matrix.h"

using namespace nemo;
using namespace boost::unit_test;

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"


BOOST_AUTO_TEST_SUITE(MatrixVectorArithmetics)


BOOST_AUTO_TEST_CASE(matrixVectorMult){
	// zero dim matrices
	RealMatrix mDim0;
	BOOST_CHECK_EQUAL( (mDim0 * RealVector()).dimension(), 0 );
	RealMatrix mDim01(dim(0,1));
	BOOST_CHECK_EQUAL( (mDim01 * RealVector(dim(1))).dimension(), 0 );
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( mDim01 * RealVector(dim(0)), std::invalid_argument );
#endif
	RealMatrix mDim10(dim(1,0));
	BOOST_CHECK_EQUAL( (mDim10 * RealVector()), RealVector(dim(1)) );
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( mDim10 * RealVector(dim(1)), std::invalid_argument );
#endif

	// square matrices
	RealMatrix m1(dim(2,2), 1.0);
	RealMatrix m1B = m1;
	BOOST_CHECK_EQUAL( (m1 * RealVector(2.0, 2.0)), RealVector(4.0, 4.0) );
	BOOST_CHECK_EQUAL( m1B, RealMatrix(dim(2,2), 1.0) );
	
	RealMatrix m2(dim(2,2));
	m2(0,0) = 2.0; m2(0,1) = 3.0; m2(1,0) = 5.0; m2(1,1) = 7.0;
	BOOST_CHECK_EQUAL( (m2 * RealVector(11.0, 13.0)), RealVector(2.0*11+3*13, 5.0*11+7*13) );

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( m1 * RealVector(dim(1)), std::invalid_argument );
#endif

	// rect matrices
	RealMatrix m3(dim(1,2), 1.0);
	RealMatrix m3B = m3;
	BOOST_CHECK_EQUAL( (m3 * RealVector(2.0, 3.0)), RealVector(5.0) );
	BOOST_CHECK_EQUAL( m3B, RealMatrix(dim(1,2), 1.0) );

#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( m3 * RealVector(dim(1)), std::invalid_argument );
	BOOST_CHECK_THROW( m3 * RealVector(dim(3)), std::invalid_argument );
#endif
}

BOOST_AUTO_TEST_CASE(matrixVectorMultSelfAssign){
	
	RealMatrix m(dim(2,2));
	m(0,0) = 2.0; m(0,1) = 3.0; m(1,0) = 5.0; m(1,1) = 7.0;
	RealVector v(11.0, 13.0);
	RealVector vB = v;
	v = m * v;
	RealVector r(2.0*11+3*13, 5.0*11+7*13);
	BOOST_CHECK_EQUAL( v, r );
}

BOOST_AUTO_TEST_SUITE_END()

