
#define BOOST_TEST_MODULE Norms
#include <boost/test/unit_test.hpp>

#include "../src/nemo/Norms.h"
#include "../src/nemo/Vector.h"
#include "../src/nemo/Matrix.h"

#include <iostream>

using namespace nemo;
using namespace boost::unit_test;

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

typedef Mapping<double,double> RealMapping;
//typedef Mapping<int,int> IntMapping;
//typedef Mapping<RealVector,RealVector > VectorMapping;
typedef Mapping<RealVector,double> VectorNorm;
typedef Mapping<RealMatrix,double> MatrixNorm;


BOOST_AUTO_TEST_SUITE(Norms)


BOOST_AUTO_TEST_CASE(absN){
	RealMapping absF = absNorm<double>();
	BOOST_CHECK_EQUAL(absF(-2.0), 2.0);
	BOOST_CHECK_EQUAL(absF(1.0), 1.0);
	BOOST_CHECK_EQUAL(absNorm<double>()(-2.0), 2.0);
	BOOST_CHECK_EQUAL(absNorm<double>()(1.0), 1.0);
	RealMapping absF2 = absNorm<double>()(2.0 * arg<double>());
	BOOST_CHECK_EQUAL(absF2(-2.0), 4.0);
	BOOST_CHECK_EQUAL(absF2(1.0), 2.0);
}

BOOST_AUTO_TEST_CASE(euclideanVector){
#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	VectorNorm eucl = euclideanNorm<RealVector>();
#else
	VectorNorm eucl = euclideanNorm<RealVector,double>();
#endif

	BOOST_CHECK_EQUAL(eucl(RealVector(3.0, 4.0)), 5.0);
	
	BOOST_CHECK_EQUAL( (euclideanNorm<RealVector,double>()(RealVector(3.0, 4.0))) , 5.0);
	BOOST_CHECK_EQUAL( (euclideanNorm<RealVector,double>()(RealVector(5.0, 12.0))) , 13.0);

	VectorNorm eucl2 = euclideanNorm<RealVector,double>()(2.0 * arg<RealVector>());
	BOOST_CHECK_EQUAL(eucl2(RealVector(3.0, 4.0)), 10.0);
}

BOOST_AUTO_TEST_CASE(euclideanMatrix){
	RealMatrix m1(dim(2,2));
	m1.setCol(0, RealVector(0.0, 1.0));
	m1.setCol(1, RealVector(1.0, 0.0));
	RealMatrix m2(dim(2,2));
	m2.setCol(0, RealVector(3.0, 0.0));
	m2.setCol(1, RealVector(0.0, 4.0));

#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	MatrixNorm eucl = euclideanNorm<RealMatrix>();
#else
	MatrixNorm eucl = euclideanNorm<RealMatrix,double>();
#endif
	BOOST_CHECK_EQUAL(eucl(m1), sqrt(2.0));
	BOOST_CHECK_EQUAL(eucl(m2), 5.0);
	
	BOOST_CHECK_EQUAL( (euclideanNorm<RealMatrix,double>()(m1)) , sqrt(2.0));
	BOOST_CHECK_EQUAL( (euclideanNorm<RealMatrix,double>()(m2)) , 5.0);

	MatrixNorm eucl2 = euclideanNorm<RealMatrix,double>()(2.0 * arg<RealMatrix>());
	BOOST_CHECK_EQUAL(eucl2(m2), 10.0);
}

BOOST_AUTO_TEST_CASE(euclideanVectorSquare){
#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	VectorNorm euclS = euclideanSquare<RealVector>();
#else
	VectorNorm euclS = euclideanSquare<RealVector,double>();
#endif
	BOOST_CHECK_EQUAL(euclS(RealVector(3.0, 4.0)), 25.0);
}

BOOST_AUTO_TEST_CASE(maxVector){
#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	VectorNorm max = maxNorm<RealVector>();
#else
	VectorNorm max = maxNorm<RealVector,double>();
#endif
	BOOST_CHECK_EQUAL(max(RealVector(3.0, 4.0)), 4.0);
	BOOST_CHECK_EQUAL(max(RealVector(-3.0, 4.0)), 4.0);
	BOOST_CHECK_EQUAL(max(RealVector(3.0, -4.0)), 4.0);
}



BOOST_AUTO_TEST_SUITE_END()

