
#define BOOST_TEST_MODULE ScalarMappings
#include <boost/test/unit_test.hpp>

#include <iostream>

#include "../src/nemo/ScalarMappings.h"


using namespace nemo;
using namespace boost::unit_test;

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

typedef Mapping<double,double> RealMapping;
typedef Mapping<int,int> IntMapping;

BOOST_AUTO_TEST_SUITE(ScalarMappings)


BOOST_AUTO_TEST_CASE(trigonometric){
#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	RealMapping c = Cos();
	RealMapping s = Sin();
	RealMapping t = Tan();
#else
	RealMapping c = Cos<double>();
	RealMapping s = Sin<double>();
	RealMapping t = Tan<double>();
#endif

	BOOST_CHECK_EQUAL( c(1.0), cos(1.0) );
	BOOST_CHECK_EQUAL( s(1.0), sin(1.0) );
	BOOST_CHECK_EQUAL( t(1.0), tan(1.0) );

	BOOST_CHECK_EQUAL( c.inverse()(0.5), acos(0.5) );
	BOOST_CHECK_EQUAL( s.inverse()(0.5), asin(0.5) );
	BOOST_CHECK_EQUAL( t.inverse()(0.5), atan(0.5) );
}

BOOST_AUTO_TEST_CASE(potentiation){
	RealMapping p = Pow<double,double>(3.0);
	RealMapping pi = Pow<double,int>(2);

#ifdef _NEMO_HAS_FUNCTION_TEMPLATE_DEFAULTS_
	RealMapping sqr = Sqrt();
	RealMapping e = Exp();
	RealMapping l = Log();
#else
	RealMapping sqr = Sqrt<double>();
	RealMapping e = Exp<double>();
	RealMapping l = Log<double>();
#endif

	BOOST_CHECK_EQUAL( p(2.0), 8.0 );
	BOOST_CHECK_EQUAL( pi(2.0), 4.0 );
	BOOST_CHECK_EQUAL( sqr(4.0), 2.0 );
	BOOST_CHECK_EQUAL( e(2.0), std::exp(2.0) );
	BOOST_CHECK_EQUAL( l(2.0), std::log(2.0) );

	BOOST_CHECK_EQUAL( p.inverse()(8.0), 2.0 );
	BOOST_CHECK_EQUAL( pi.inverse()(4.0), 2.0 );
	BOOST_CHECK_EQUAL( sqr.inverse()(2.0), 4.0 );
	BOOST_CHECK_EQUAL( e.inverse()(0.5), std::log(0.5) );
	BOOST_CHECK_EQUAL( l.inverse()(0.5), std::exp(0.5) );
}

BOOST_AUTO_TEST_CASE(absValue){
	RealMapping fa = Abs<double>();
	BOOST_CHECK_EQUAL( fa(-1.0), 1.0 );
	BOOST_CHECK_EQUAL( fa( 0.0), 0.0 );
	BOOST_CHECK_EQUAL( fa( 1.0), 1.0 );

	IntMapping ia = Abs<int>();
	BOOST_CHECK_EQUAL( ia(-1 ), 1 );
	BOOST_CHECK_EQUAL( ia( 0 ), 0 );
	BOOST_CHECK_EQUAL( ia( 1 ), 1 );
}

BOOST_AUTO_TEST_CASE(modulo){
	RealMapping fm = Mod<double>(2.0);
	BOOST_CHECK_EQUAL( fm(-3.0), -1.0 );
	BOOST_CHECK_EQUAL( fm( 1.0),  1.0 );
	BOOST_CHECK_EQUAL( fm( 1.5),  1.5 );

	RealMapping fm2 = Mod<double>(7.0);
	BOOST_CHECK_EQUAL( fm2(-3.0), -3.0 );
	BOOST_CHECK_EQUAL( fm2( 1.0),  1.0 );
	BOOST_CHECK_EQUAL( fm2( 1.5),  1.5 );

	IntMapping im = Mod<int>(7);
	BOOST_CHECK_EQUAL( im(-1 ), -1 );
	BOOST_CHECK_EQUAL( im( 1 ), 1 );
	BOOST_CHECK_EQUAL( im(10 ), 3 );
}


BOOST_AUTO_TEST_SUITE_END()

