
#define BOOST_TEST_MODULE TimeSeriesIterator
#include <boost/test/unit_test.hpp>

#include "../src/nemo/TimeSeries.h"


// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE(TimeSeriesIterator)


BOOST_AUTO_TEST_CASE(viewIterator) {
	TimeSeries<int,double> r1;
		
	BOOST_CHECK(r1.timestamps.begin() == r1.timestamps.end());
	BOOST_CHECK(r1.values.begin() == r1.values.end());
	
	r1.append(0, 1.0);
	r1.append(1, 2.0);
	r1.append(2, 3.0);

	TimeSeries<int,double>::ViewIterator<int> itT = r1.timestamps.begin();
	BOOST_CHECK_EQUAL(*itT, 0);
	++itT;
	BOOST_CHECK_EQUAL(*itT, 1);
	++itT;
	BOOST_CHECK_EQUAL(*itT, 2);
	++itT;
	BOOST_CHECK(itT == r1.timestamps.end());
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( *itT, std::logic_error );
#endif

	TimeSeries<int,double>::ViewIterator<double> itV = r1.values.begin();
	BOOST_CHECK_EQUAL(*itV, 1.0);
	++itV;
	BOOST_CHECK_EQUAL(*itV, 2.0);
	++itV;
	BOOST_CHECK_EQUAL(*itV, 3.0);
	++itV;
	BOOST_CHECK(itV == r1.values.end());
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( *itV, std::logic_error );
#endif
}

BOOST_AUTO_TEST_CASE(viewIteratorPostfix) {
	TimeSeries<int,double> r1;
	r1.append(0, 1.0);
	r1.append(1, 2.0);
	r1.append(2, 3.0);

	TimeSeries<int,double>::ViewIterator<int> itT = r1.timestamps.begin();
	BOOST_CHECK_EQUAL( *(itT++), 0 );
	BOOST_CHECK_EQUAL( *(itT++), 1 );
	BOOST_CHECK_EQUAL( *(itT++), 2 );
	BOOST_CHECK(itT == r1.timestamps.end());

	TimeSeries<int,double>::ViewIterator<double> itV = r1.values.begin();
	BOOST_CHECK_EQUAL( *(itV++), 1.0 );
	BOOST_CHECK_EQUAL( *(itV++), 2.0 );
	BOOST_CHECK_EQUAL( *(itV++), 3.0 );
	BOOST_CHECK(itV == r1.values.end());
}

BOOST_AUTO_TEST_CASE(viewIteratorStlRead) {
	TimeSeries<int,double> r1;
	r1.append(0, 1.0);
	r1.append(1, 2.0);
	r1.append(2, 3.0);

	BOOST_CHECK_EQUAL( std::count(r1.timestamps.begin(), r1.timestamps.end(), 2), 1 );
	BOOST_CHECK_EQUAL( *std::min_element(r1.timestamps.begin(), r1.timestamps.end()), 0 );
	BOOST_CHECK_EQUAL( *std::max_element(r1.timestamps.begin(), r1.timestamps.end()), 2 );

	BOOST_CHECK_EQUAL( std::count(r1.values.begin(), r1.values.end(), 2.0), 1 );
	BOOST_CHECK_EQUAL( *std::min_element(r1.values.begin(), r1.values.end()), 1.0 );
	BOOST_CHECK_EQUAL( *std::max_element(r1.values.begin(), r1.values.end()), 3.0 );
}

BOOST_AUTO_TEST_CASE(entryIterator) {
	TimeSeries<int,double> r1;
	typedef TimeSeries<int,double>::EntryType EntryType;
		
	BOOST_CHECK(r1.begin() == r1.end());
	
	r1.append(0, 1.0);
	r1.append(1, 2.0);
	r1.append(2, 3.0);

	TimeSeries<int,double>::EntryIterator it = r1.begin();
	BOOST_CHECK_EQUAL(*it, EntryType( 0, 1.0 ));
	++it;
	BOOST_CHECK_EQUAL(*it, EntryType( 1, 2.0 ));
	++it;
	BOOST_CHECK_EQUAL(*it, EntryType( 2, 3.0 ));
	++it;
	BOOST_CHECK(it == r1.end());
#ifndef NEMO_ABORT_ON_ERROR
	BOOST_CHECK_THROW( *it, std::logic_error );
#endif
}

BOOST_AUTO_TEST_CASE(entryIteratorStlRead) {
	TimeSeries<int,double> r1;
	typedef TimeSeries<int,double>::EntryType EntryType;
	r1.append(0, 1.0);
	r1.append(1, 2.0);
	r1.append(2, 3.0);

	BOOST_CHECK_EQUAL( std::count(r1.begin(), r1.end(), EntryType( 0, 2.0 )), 0 );
	BOOST_CHECK_EQUAL( std::count(r1.begin(), r1.end(), EntryType( 1, 2.0 )), 1 );
}


BOOST_AUTO_TEST_SUITE_END()


