
#define BOOST_TEST_MODULE GaussianRandom
#include <boost/test/unit_test.hpp>

#include "../../src/nemo/Random.h"

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
//using namespace boost::unit_test; // confuses MSVC when resolving std::log vs boost::unit_test::log


BOOST_AUTO_TEST_SUITE(GaussianRamdomGenerator)


BOOST_AUTO_TEST_CASE(gaussianNormal) {
	gaussian<double> g;
	
	long double avgPairMean = 0.0;
	long double avgPairMoment2 = 0.0;
	long double avgPairMoment3 = 0.0;
	long double avgPairMoment4 = 0.0;
	long double avgPairCorr = 0.0;
	
	unsigned int N = 500000;
	for(unsigned int i=0; i<N; i++){
		double x1 = g();
		double x2 = g();
		
		avgPairMean += (x1+x2)/2.0;
		avgPairMoment2 += ((x1*x1)+(x2*x2))/2.0;
		avgPairMoment3 += ((x1*x1*x1)+(x2*x2*x2))/2.0;
		avgPairMoment4 += ((x1*x1*x1*x1)+(x2*x2*x2*x2))/2.0;
		
		avgPairCorr += x1*x2;
	}
	avgPairMean /= N;
	avgPairMoment2 /= N;
	avgPairMoment3 /= N;
	avgPairMoment4 /= N;

	avgPairCorr /= N;

	// does the generator hit the desired distribution?
	BOOST_CHECK_SMALL( (double)avgPairMean, 1e-2 );
	BOOST_CHECK_SMALL( (double)avgPairMoment2-1, 1e-2 );
	BOOST_CHECK_SMALL( (double)avgPairMoment3-0, 1e-2 );
	BOOST_CHECK_SMALL( (double)avgPairMoment4-3, 1e-2 );

	// are successive values independent???
	BOOST_CHECK_SMALL( (double)avgPairCorr, 1e-2 );
}

BOOST_AUTO_TEST_CASE(gaussianParam) {
	gaussian<double> g(2.0, 42.0);
	
	long double mean = 0.0;
	long double stdDev = 0.0;
	
	unsigned int N = 100000;
	for(unsigned int i=0; i<N; i++){
		double x = g();
		
		mean += x;
		stdDev += (x-42)*(x-42);
	}
	mean /= N;
	stdDev = sqrt(stdDev/N);

	// does the generator hit the desired distribution?
	BOOST_CHECK_SMALL( (double)mean-42, 1.1e-2 );
	BOOST_CHECK_SMALL( (double)stdDev-2, 1e-2 );
}


BOOST_AUTO_TEST_CASE(gaussianCopyConstruct) {
	gaussian<double> g1;
	BOOST_CHECK( g1() != 0.0 );
	gaussian<double> g2 = g1;

	BOOST_CHECK( g1() != g2() );
	
}


BOOST_AUTO_TEST_SUITE_END()

