
#define BOOST_TEST_MODULE UniformRandom
#include <boost/test/unit_test.hpp>

#include "../../src/nemo/Random.h"

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"

using namespace nemo;
using namespace boost::unit_test;


BOOST_AUTO_TEST_SUITE()



BOOST_AUTO_TEST_CASE(uniformInt) {
	//BOOST_REQUIRE(uniform<int>::isIntegral);

	const unsigned int N = 10;
	unsigned int bins[N];
	for(unsigned int i=0; i<N; i++) bins[i] = 0;

	uniform<int> u1(0,N-1);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		int x = u1();
		if( x < 0 ){ tooLow = true; break; }
		if( (int)N-1 < x ){ tooHigh = true; break; }
		bins[x]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[0]);
	// check intermediate values
	for(unsigned int i=1; i<N-1; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[N-1]);
}
BOOST_AUTO_TEST_CASE(uniformIntSingular) {
	uniform<int> u(0,0);
	for(unsigned int i=0; i<100; i++){
		int x = u();
		BOOST_REQUIRE_EQUAL(x, 0);
	}
}
BOOST_AUTO_TEST_CASE(uniformIntNegRange) {
	const unsigned int N = 10;
	unsigned int bins[N];
	for(unsigned int i=0; i<N; i++) bins[i] = 0;

	uniform<int> u1(N-1,0);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		int x = u1();
		if( x < 0 ){ tooLow = true; break; }
		if( (int)N-1 < x ){ tooHigh = true; break; }
		bins[x]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[0]);
	// check intermediate values
	for(unsigned int i=1; i<N-1; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[N-1]);
}

BOOST_AUTO_TEST_CASE(uniformUInt) {
	//BOOST_REQUIRE(uniform<unsigned int>::isIntegral);
	
	const unsigned int N = 10;
	unsigned int bins[N+1];
	for(unsigned int i=0; i<=N; i++) bins[i] = 0;

	uniform<unsigned int> u1(1,N);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		unsigned int x = u1();
		if( x < 1 ){ tooLow = true; break; }
		if( (int)N < x ){ tooHigh = true; break; }
		bins[x]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[1]);
	// check intermediate values
	for(unsigned int i=1; i<N; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[N]);
}
BOOST_AUTO_TEST_CASE(uniformUIntSingular) {
	uniform<unsigned int> u(0,0);
	for(unsigned int i=0; i<100; i++){
		unsigned int x = u();
		BOOST_REQUIRE_EQUAL(x, 0);
	}
}
BOOST_AUTO_TEST_CASE(uniformUIntNegRange) {
	const unsigned int N = 10;
	unsigned int bins[N+1];
	for(unsigned int i=0; i<=N; i++) bins[i] = 0;

	uniform<unsigned int> u1(N,1);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		unsigned int x = u1();
		if( x < 1 ){ tooLow = true; break; }
		if( (int)N < x ){ tooHigh = true; break; }
		bins[x]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[1]);
	// check intermediate values
	for(unsigned int i=1; i<N; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[N]);
}

BOOST_AUTO_TEST_CASE(uniformDouble) {
	//BOOST_REQUIRE( !(uniform<double>::isIntegral) );
	
	const double min = 1.0;
	const double max = 2.0;
	const unsigned int numBins = 8;
	unsigned int bins[numBins];
	for(unsigned int i=0; i<numBins; i++) bins[i] = 0;

	uniform<double> u1(min,max);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		double x = u1();
		if( x < min ){ tooLow = true; break; }
		if( max < x ){ tooHigh = true; break; }
		unsigned int binIndex = (x-min)/(max-min) * numBins;
		assert(binIndex < numBins);
		bins[binIndex]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[0]);
	// check intermediate values
	for(unsigned int i=1; i<numBins-1; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[numBins-1]);
}
BOOST_AUTO_TEST_CASE(uniformDoubleSingular) {
	uniform<double> u(0,0);
	for(unsigned int i=0; i<100; i++){
		double x = u();
		BOOST_REQUIRE_EQUAL(x, 0.0);
	}
}
BOOST_AUTO_TEST_CASE(uniformDoubleNegRange) {
	const double min = 1.0;
	const double max = 2.0;
	const unsigned int numBins = 8;
	unsigned int bins[numBins];
	for(unsigned int i=0; i<numBins; i++) bins[i] = 0;

	uniform<double> u1(max, min);
	bool tooLow = false;
	bool tooHigh = false;
	for(unsigned int i=0; i<10000; i++){
		double x = u1();
		if( x < min ){ tooLow = true; break; }
		if( max < x ){ tooHigh = true; break; }
		unsigned int binIndex = (x-min)/(max-min) * numBins;
		assert(binIndex < numBins);
		bins[binIndex]++;
	}
	BOOST_REQUIRE( !tooLow );
	BOOST_REQUIRE( !tooHigh );
	// check min
	BOOST_CHECK(0 < bins[0]);
	// check intermediate values
	for(unsigned int i=1; i<numBins-1; i++){
		BOOST_CHECK(0 < bins[i]);
	}
	// check max
	BOOST_CHECK(0 < bins[numBins-1]);
}

BOOST_AUTO_TEST_SUITE_END()

